/****************************************************************
* SOUNDEX.C - a program to demonstrate soundex codes			*
* To compile "cl soundex.c"										*
****************************************************************/

#include <stdio.h>
#include <ctype.h>

/****************************************************************
* soundex - return the soundex code for an input string			*
*																*
* INP:	input - the word to compress to a soundex code			*
*		output - the buffer to hold the soundex code			*
* OUT:	output buffer modified (assumed to hold least 5 char)	*
* NOTE:	only letters are used in soundex, any non-letter will	*
*		terminate the word.										*
****************************************************************/
static char sxData[26]=
	{
/*	 A,  B,  C,  D,  E,  F,  G,  H,  I,  J,  K,  L,  M,	*/
	 0, '1','2','3', 0, '1','2', 0,  0, '2','2','4','5',
/*	 N,  O,  P,  Q,  R,  S,  T,  U,  V,  W,  X,  Y,  Z	*/
	'5', 0, '1','2','6','2','3', 0, '1', 0, '2', 0, '2'
	};
char *soundex(char *input, char *output)
	{
	int i, o;						/* Inp & out buffer indices	*/
	int c;							/* Temporary value			*/
	for (i=o=0; o<4; i++)
		{
		c = input[i];				/* Get next character		*/

		/* End of input string? */
		if (!isalpha(c))			/* Is it not a letter?		*/
			{						/* It isn't ... terminate	*/
			if (o==0)				/* Is soundex buffer empty?	*/
				output[0]=0;		/* Yes, return empty string	*/
			else					/* No, pad buffer with '0'	*/
				{
				while (o<4)
					output[o++]='0';
				output[o]=0;		/* Terminate string			*/
				}
			return output;			/* Return to caller			*/
			}

		c = toupper(c);				/* Map to upper case		*/
		if (o==0)					/* Is it the first char?	*/
			output[o++]=(char)c;	/* Y: store in output buff	*/
		else
			{
			c = sxData[c-'A'];			/* Look up code fm table*/
			if (c)						/* Ignore character?	*/
				if (output[o-1]!=(char)c) /* Same as last one?	*/
					output[o++]=(char)c;  /* No, put in out buf */
			}
		}
	return output;
	}

/****************************************************************
* main - test function for soundex.  Prompt user for word, and	*
* print the resulting soundex code.								*
****************************************************************/
void main(void)
	{
	char inline[200], output[6];
	do	{
		puts("\nEnter a word or press ENTER to quit:");
		gets(inline);
		soundex(inline,output);
		printf("The soundex code for \'%s\' is \'%s\'\n",
									inline,		output);
		} while (inline[0]);
	}
